@@ -56,7 +56,7 @@ TagsCtx = AbstractSet[str]
 VolumesCtx = Mapping[str, "VolumeCtx"]
 ImagesCtx = Mapping[str, "ImageCtx"]
 InputsCtx = Mapping[str, str]
-ArgsCtx = Mapping[str, str]
+ParamsCtx = Mapping[str, str]
 
 NeedsCtx = Mapping[str, "DepCtx"]
 StateCtx = Mapping[str, str]
@@ -278,20 +278,39 @@ class LiveContext(WithEnvContext, LiveContextStep1):
     volumes: VolumesCtx
     images: ImagesCtx
 
-    def to_multi_ctx(self, multi: MultiCtx) -> "LiveContextMulti":
-        return LiveContextMulti(
+    def to_job_ctx(self, params: ParamsCtx) -> "LiveJobContext":
+        return LiveJobContext(
+            flow=self.flow,
+            env=self.env,
+            tags=self.tags,
+            volumes=self.volumes,
+            images=self.images,
+            params=params,
+        )
+
+    def to_multi_job_ctx(
+        self, multi: MultiCtx, params: ParamsCtx
+    ) -> "LiveMultiJobContext":
+        return LiveMultiJobContext(
             flow=self.flow,
             env=self.env,
             tags=self.tags,
             volumes=self.volumes,
             images=self.images,
             multi=multi,
+            params=params,
         )
 
 
 @dataclass(frozen=True)
-class LiveContextMulti(LiveContext):
+class LiveJobContext(LiveContext):
+    params: ParamsCtx
+
+
+@dataclass(frozen=True)
+class LiveMultiJobContext(LiveContext):
     multi: MultiCtx
+    params: ParamsCtx
 
 
 @dataclass(frozen=True)
@@ -302,14 +321,14 @@ class LiveActionContext(WithEnvContext, Context):
 
 @dataclass(frozen=True)
 class BatchContextStep1(WithFlowContext, Context):
-    args: ArgsCtx
+    params: ParamsCtx
 
     def to_step_2(
         self, env: EnvCtx, tags: TagsCtx, volumes: VolumesCtx, images: ImagesCtx
     ) -> "BatchContextStep2":
         return BatchContextStep2(
             flow=self.flow,
-            args=self.args,
+            params=self.params,
             env=env,
             tags=tags,
             volumes=volumes,
@@ -329,7 +348,7 @@ class BatchContextStep2(WithEnvContext, BatchContextStep1):
     ) -> "BatchContext":
         return BatchContext(
             flow=self.flow,
-            args=self.args,
+            params=self.params,
             env=self.env,
             tags=self.tags,
             volumes=self.volumes,
@@ -357,6 +376,11 @@ class BaseMatrixContext(BaseBatchContext):
         pass
 
 
+@dataclass(frozen=True)
+class MatrixOnlyContext(Context):
+    matrix: MatrixCtx
+
+
 class BaseTaskContext(BaseMatrixContext):
     strategy: StrategyCtx
     needs: NeedsCtx
@@ -372,7 +396,7 @@ class BatchContext(BaseBatchContext, BatchContextStep2):
     ) -> "BatchMatrixContext":
         return BatchMatrixContext(
             flow=self.flow,
-            args=self.args,
+            params=self.params,
             env=self.env,
             tags=self.tags,
             volumes=self.volumes,
@@ -389,7 +413,7 @@ class BatchMatrixContext(BaseMatrixContext, BatchContext):
     def to_task_ctx(self, needs: NeedsCtx, state: StateCtx) -> "BatchTaskContext":
         return BatchTaskContext(
             flow=self.flow,
-            args=self.args,
+            params=self.params,
             env=self.env,
             tags=self.tags,
             volumes=self.volumes,
@@ -645,22 +669,30 @@ async def setup_inputs_ctx(
     return new_inputs
 
 
-async def setup_args_ctx(
+async def setup_params_ctx(
     ctx: RootABC,
-    ast_args: Optional[Mapping[str, ast.Arg]],
-) -> ArgsCtx:
-    args = {}
-    if ast_args is not None:
-        for k, v in ast_args.items():
-            default = await v.default.eval(ctx)
-            if default is None:
+    params: Optional[Mapping[str, str]],
+    ast_params: Optional[Mapping[str, ast.Param]],
+) -> ParamsCtx:
+    if params is None:
+        params = {}
+    new_params = {}
+    if ast_params is not None:
+        for k, v in ast_params.items():
+            value = params.get(k) or await v.default.eval(ctx)
+            if value is None:
                 raise EvalError(
                     f"Arg {k} is not initialized and has no default value",
                     v._start,
                     v._end,
                 )
-            args[k] = default
-    return args
+            new_params[k] = value
+    extra = params.keys() - new_params.keys()
+    if extra:
+        raise ValueError(
+            f"Unsupported arg(s): {','.join(sorted(extra))}",
+        )
+    return new_params
 
 
 async def setup_strategy_ctx(
@@ -678,6 +710,45 @@ async def setup_strategy_ctx(
     return StrategyCtx(fail_fast=fail_fast, max_parallel=max_parallel)
 
 
+async def setup_matrix(ast_matrix: Optional[ast.Matrix]) -> Sequence[MatrixCtx]:
+    if ast_matrix is None:
+        return [{}]
+    # Init
+    products = []
+    for k, lst in ast_matrix.products.items():
+        lst2 = [{k: await i.eval(EMPTY_ROOT)} for i in lst]
+        products.append(lst2)
+    matrices = []
+    for row in itertools.product(*products):
+        dct: Dict[str, LiteralT] = {}
+        for elem in row:
+            dct.update(elem)
+        matrices.append(dct)
+    # Exclude
+    exclude = []
+    for exc_spec in ast_matrix.exclude:
+        exclude.append({k: await v.eval(EMPTY_ROOT) for k, v in exc_spec.items()})
+    filtered = []
+    for matrix in matrices:
+        include = True
+        for exc in exclude:
+            match = True
+            for k, v in exc.items():
+                if matrix[k] != v:
+                    match = False
+                    break
+            if match:
+                include = False
+                break
+        if include:
+            filtered.append(matrix)
+    matrices = filtered
+    # Include
+    for inc_spec in ast_matrix.include:
+        matrices.append({k: await v.eval(EMPTY_ROOT) for k, v in inc_spec.items()})
+    return matrices
+
+
 async def setup_cache(
     ctx: RootABC,
     base_cache: CacheConf,
@@ -772,14 +843,22 @@ class RunningLiveFlow:
             tags=tags,
         )
 
-    async def get_job(self, job_id: str) -> Job:
+    async def get_job(self, job_id: str, params: Mapping[str, str]) -> Job:
         assert not await self.is_multi(
             job_id
         ), "Use get_multi_job() for multi jobs instead of get_job()"
-        return await self._get_job(self._ctx, self._defaults, job_id)
+        job_ast = self._get_job_ast(job_id)
+        ctx = self._ctx.to_job_ctx(
+            params=await setup_params_ctx(EMPTY_ROOT, params, job_ast.params)
+        )
+        return await self._get_job(ctx, self._defaults, job_id, params)
 
     async def get_multi_job(
-        self, job_id: str, suffix: str, args: Optional[Sequence[str]]
+        self,
+        job_id: str,
+        suffix: str,
+        args: Optional[Sequence[str]],
+        params: Mapping[str, str],
     ) -> Job:
         assert await self.is_multi(
             job_id
@@ -789,14 +868,20 @@ class RunningLiveFlow:
             args_str = ""
         else:
             args_str = " ".join(shlex.quote(arg) for arg in args)
-        ctx = self._ctx.to_multi_ctx(
+        job_ast = self._get_job_ast(job_id)
+        ctx = self._ctx.to_multi_job_ctx(
             multi=MultiCtx(suffix=suffix, args=args_str),
+            params=await setup_params_ctx(EMPTY_ROOT, params, job_ast.params),
         )
-        job = await self._get_job(ctx, self._defaults, job_id)
+        job = await self._get_job(ctx, self._defaults, job_id, params)
         return replace(job, tags=job.tags | {f"multi:{suffix}"})
 
     async def _get_job(
-        self, ctx: WithEnvContext, defaults: DefaultsConf, job_id: str
+        self,
+        ctx: WithEnvContext,
+        defaults: DefaultsConf,
+        job_id: str,
+        params: Mapping[str, str],
     ) -> Job:
         job = self._get_job_ast(job_id)
         if isinstance(job, ast.JobActionCall):
@@ -890,18 +975,12 @@ class RunningLiveFlow:
 _T = TypeVar("_T", bound=BaseBatchContext, covariant=True)
 
 
-class RunningBatchBase(Generic[_T]):
+class EarlyBatch:
     def __init__(
-        self,
-        ctx: _T,
-        tasks: Mapping[str, "BasePrepTask"],
-        config_loader: ConfigLoader,
-        defaults: DefaultsConf,
+        self, tasks: Mapping[str, "BaseEarlyTask"], config_loader: ConfigLoader
     ):
-        self._ctx = ctx
-        self._tasks: Mapping[str, BasePrepTask] = tasks
         self._cl = config_loader
-        self._defaults = defaults
+        self._tasks = tasks
 
     @property
     def graph(self) -> Mapping[str, AbstractSet[str]]:
@@ -910,32 +989,63 @@ class RunningBatchBase(Generic[_T]):
     @lru_cache()
     def _graph(self) -> Mapping[str, AbstractSet[str]]:
         # This function is only needed for mypy
-        return {key: prep_task.needs for key, prep_task in self._tasks.items()}
+        return {key: early_task.needs for key, early_task in self._tasks.items()}
 
-    def _get_prep(self, real_id: str) -> "BasePrepTask":
+    def _get_prep(self, real_id: str) -> "BaseEarlyTask":
         try:
             return self._tasks[real_id]
         except KeyError:
             raise UnknownTask(real_id)
 
     async def is_task(self, real_id: str) -> bool:
-        prep_task = self._get_prep(real_id)
-        return isinstance(prep_task, (PrepTask, PrepStatefulCall))
+        early_task = self._get_prep(real_id)
+        return isinstance(early_task, EarlyTask)
 
     async def is_local(self, real_id: str) -> bool:
-        prep_task = self._get_prep(real_id)
-        return isinstance(prep_task, PrepLocalCall)
+        early_task = self._get_prep(real_id)
+        return isinstance(early_task, EarlyLocalCall)
 
     async def is_action(self, real_id: str) -> bool:
-        prep_task = self._get_prep(real_id)
-        return isinstance(prep_task, PrepBatchCall)
+        early_task = self._get_prep(real_id)
+        return isinstance(early_task, EarlyBatchCall)
 
     async def state_from(self, real_id: str) -> Optional[str]:
         prep_task = self._get_prep(real_id)
-        if isinstance(prep_task, PrepPostTask):
+        if isinstance(prep_task, EarlyPostTask):
             return prep_task.state_from
         return None
 
+    async def get_action_early(self, real_id: str) -> "EarlyBatch":
+        assert await self.is_action(
+            real_id
+        ), f"get_task() cannot used for action call {real_id}"
+        prep_task = self._get_prep(real_id)
+        assert isinstance(prep_task, PrepBatchCall)  # Already checked
+
+        tasks = await EarlyTaskGraphBuilder(self._cl, prep_task.action.tasks).build()
+
+        return EarlyBatch(tasks, self._cl)
+
+
+class RunningBatchBase(Generic[_T], EarlyBatch):
+    _tasks: Mapping[str, "BasePrepTask"]
+
+    def __init__(
+        self,
+        ctx: _T,
+        tasks: Mapping[str, "BasePrepTask"],
+        config_loader: ConfigLoader,
+        defaults: DefaultsConf,
+    ):
+        super().__init__(tasks, config_loader)
+        self._ctx = ctx
+        self._defaults = defaults
+
+    def _get_prep(self, real_id: str) -> "BasePrepTask":
+        prep_task = super()._get_prep(real_id)
+        assert isinstance(prep_task, BasePrepTask)
+        return prep_task
+
     def _task_context(
         self, real_id: str, needs: NeedsCtx, state: StateCtx
     ) -> BaseTaskContext:
@@ -1112,17 +1222,20 @@ class RunningBatchFlow(RunningBatchBase[BatchContext]):
 
     @classmethod
     async def create(
-        cls, config_loader: ConfigLoader, batch: str
+        cls,
+        config_loader: ConfigLoader,
+        batch: str,
+        params: Optional[Mapping[str, str]] = None,
     ) -> "RunningBatchFlow":
         ast_flow = await config_loader.fetch_flow(batch)
 
         assert isinstance(ast_flow, ast.BatchFlow)
 
         flow_ctx = await setup_flow_ctx(EMPTY_ROOT, ast_flow, batch, config_loader)
-        args_ctx = await setup_args_ctx(EMPTY_ROOT, ast_flow.args)
+        params_ctx = await setup_params_ctx(EMPTY_ROOT, params, ast_flow.params)
         step_1_ctx = BatchContextStep1(
             flow=flow_ctx,
-            args=args_ctx,
+            params=params_ctx,
         )
 
         defaults, env, tags = await setup_defaults_env_tags_ctx(
@@ -1232,15 +1345,125 @@ class RunningBatchActionFlow(RunningBatchBase[BatchActionContext]):
 
 
 @dataclass(frozen=True)
-class BasePrepTask:
+class BaseEarlyTask:
     id: Optional[str]
     real_id: str
     needs: AbstractSet[str]  # A set of batch.id
 
     matrix: MatrixCtx
+    enable: EnableExpr
+
+    def to_task(self, ast_task: ast.ExecUnit) -> "EarlyTask":
+        return EarlyTask(
+            id=self.id,
+            real_id=self.real_id,
+            needs=self.needs,
+            matrix=self.matrix,
+            enable=self.enable,
+            ast_task=ast_task,
+        )
+
+    def to_batch_call(
+        self,
+        action: ast.BatchAction,
+        call: ast.TaskActionCall,
+    ) -> "EarlyBatchCall":
+        return EarlyBatchCall(
+            id=self.id,
+            real_id=self.real_id,
+            needs=self.needs,
+            matrix=self.matrix,
+            enable=self.enable,
+            action=action,
+            call=call,
+        )
+
+    def to_local_call(
+        self,
+        action: ast.LocalAction,
+        call: ast.TaskActionCall,
+    ) -> "EarlyLocalCall":
+        return EarlyLocalCall(
+            id=self.id,
+            real_id=self.real_id,
+            needs=self.needs,
+            matrix=self.matrix,
+            enable=self.enable,
+            action=action,
+            call=call,
+        )
+
+    def to_stateful_call(
+        self,
+        action: ast.StatefulAction,
+        call: ast.TaskActionCall,
+    ) -> "EarlyStatefulCall":
+        return EarlyStatefulCall(
+            id=self.id,
+            real_id=self.real_id,
+            needs=self.needs,
+            matrix=self.matrix,
+            ast_task=action.main,
+            enable=self.enable,
+            action=action,
+            call=call,
+        )
+
+    def to_post_task(self, ast_task: ast.ExecUnit, state_from: str) -> "EarlyPostTask":
+        return EarlyPostTask(
+            id=self.id,
+            real_id=self.real_id,
+            needs=self.needs,
+            matrix=self.matrix,
+            enable=self.enable,
+            ast_task=ast_task,
+            state_from=state_from,
+        )
+
+    def to_prep_base(self, strategy: StrategyCtx, cache: CacheConf) -> "BasePrepTask":
+        return BasePrepTask(
+            id=self.id,
+            real_id=self.real_id,
+            needs=self.needs,
+            matrix=self.matrix,
+            enable=self.enable,
+            strategy=strategy,
+            cache=cache,
+        )
+
+
+@dataclass(frozen=True)
+class EarlyTask(BaseEarlyTask):
+    ast_task: ast.ExecUnit
+
+
+@dataclass(frozen=True)
+class EarlyBatchCall(BaseEarlyTask):
+    call: ast.TaskActionCall
+    action: ast.BatchAction
+
+
+@dataclass(frozen=True)
+class EarlyLocalCall(BaseEarlyTask):
+    call: ast.TaskActionCall
+    action: ast.LocalAction
+
+
+@dataclass(frozen=True)
+class EarlyStatefulCall(EarlyTask):
+    call: ast.TaskActionCall
+    action: ast.StatefulAction
+
+
+@dataclass(frozen=True)
+class EarlyPostTask(EarlyTask):
+    state_from: str
+
+
+@dataclass(frozen=True)
+class BasePrepTask(BaseEarlyTask):
     strategy: StrategyCtx
     cache: CacheConf
-    enable: EnableExpr
 
     def to_task(self, ast_task: ast.ExecUnit) -> "PrepTask":
         return PrepTask(
@@ -1321,60 +1544,57 @@ class BasePrepTask:
 
 
 @dataclass(frozen=True)
-class PrepTask(BasePrepTask):
-    ast_task: ast.ExecUnit
+class PrepTask(EarlyTask, BasePrepTask):
+    pass
 
 
 @dataclass(frozen=True)
-class PrepBatchCall(BasePrepTask):
-    call: ast.TaskActionCall
-    action: ast.BatchAction
+class PrepBatchCall(EarlyBatchCall, BasePrepTask):
+    pass
 
 
 @dataclass(frozen=True)
-class PrepLocalCall(BasePrepTask):
-    call: ast.TaskActionCall
-    action: ast.LocalAction
+class PrepLocalCall(EarlyLocalCall, BasePrepTask):
+    pass
 
 
 @dataclass(frozen=True)
-class PrepStatefulCall(PrepTask):
-    call: ast.TaskActionCall
-    action: ast.StatefulAction
+class PrepStatefulCall(EarlyStatefulCall, PrepTask):
+    pass
 
 
 @dataclass(frozen=True)
-class PrepPostTask(PrepTask):
-    state_from: str
+class PrepPostTask(EarlyPostTask, PrepTask):
+    pass
 
 
-class TaskGraphBuilder:
+class EarlyTaskGraphBuilder:
     MATRIX_SIZE_LIMIT = 256
 
     def __init__(
         self,
-        ctx: BaseBatchContext,
         config_loader: ConfigLoader,
-        default_cache: CacheConf,
         ast_tasks: Sequence[Union[ast.Task, ast.TaskActionCall]],
     ):
-        self._ctx = ctx
         self._cl = config_loader
-        self._default_cache = default_cache
         self._ast_tasks = ast_tasks
 
-    async def build(self) -> Mapping[str, BasePrepTask]:
-        post_tasks: List[List[BasePrepTask]] = []
-        prep_tasks: Dict[str, BasePrepTask] = {}
+    async def _extend_base(
+        self, base: BaseEarlyTask, ast_task: Union[ast.Task, ast.TaskActionCall]
+    ) -> BaseEarlyTask:
+        return base
+
+    async def build(self) -> Mapping[str, BaseEarlyTask]:
+        post_tasks: List[List[EarlyPostTask]] = []
+        prep_tasks: Dict[str, BaseEarlyTask] = {}
         last_needs: Set[str] = set()
         for num, ast_task in enumerate(self._ast_tasks, 1):
             assert isinstance(ast_task, (ast.Task, ast.TaskActionCall))
 
-            strategy = await self._setup_strategy(ast_task.strategy)
             matrix_ast = ast_task.strategy.matrix if ast_task.strategy else None
-            matrix = await self._setup_matrix(matrix_ast)
+            matrices = await setup_matrix(matrix_ast)
 
-            if len(matrix) > TaskGraphBuilder.MATRIX_SIZE_LIMIT:
+            if len(matrices) > self.MATRIX_SIZE_LIMIT:
                 assert matrix_ast
                 raise EvalError(
                     f"The matrix size for task #{num} exceeds the limit of 256",
@@ -1383,29 +1603,22 @@ class TaskGraphBuilder:
                 )
 
             real_ids = set()
-            post_tasks_group: List[BasePrepTask] = []
-            for row in matrix:
+            post_tasks_group = []
+            for matrix in matrices:
                 # make prep patch(es)
-                matrix_ctx = self._ctx.to_matrix_ctx(matrix=row, strategy=strategy)
+                matrix_ctx = MatrixOnlyContext(matrix=matrix)
 
                 task_id, real_id = await self._setup_ids(matrix_ctx, num, ast_task)
                 needs = await self._setup_needs(matrix_ctx, last_needs, ast_task)
-                cache = await setup_cache(
-                    matrix_ctx,
-                    self._default_cache,
-                    ast_task.cache,
-                    ast.CacheStrategy.INHERIT,
-                )
 
-                base = BasePrepTask(
+                base = BaseEarlyTask(
                     id=task_id,
                     real_id=real_id,
                     needs=needs,
-                    matrix=row,
-                    strategy=strategy,
-                    cache=cache,
+                    matrix=matrix,
                     enable=ast_task.enable,
                 )
+                base = await self._extend_base(base, ast_task)
 
                 if isinstance(ast_task, ast.Task):
                     prep_tasks[real_id] = base.to_task(ast_task)
@@ -1418,7 +1631,6 @@ class TaskGraphBuilder:
                     elif isinstance(action, ast.LocalAction):
                         prep_tasks[real_id] = base.to_local_call(action, ast_task)
                     elif isinstance(action, ast.StatefulAction):
-                        prep_tasks[real_id] = base.to_stateful_call(action, ast_task)
                         if action.post:
                             post_tasks_group.append(
                                 replace(
@@ -1429,6 +1641,8 @@ class TaskGraphBuilder:
                                     enable=action.post_if,
                                 ).to_post_task(action.post, real_id),
                             )
+                        prep_tasks[real_id] = base.to_stateful_call(action, ast_task)
+
                     else:
                         raise ValueError(
                             f"Action {action_name} has kind {action.kind}, "
@@ -1451,77 +1665,8 @@ class TaskGraphBuilder:
 
         return prep_tasks
 
-    async def _setup_strategy(
-        self, ast_strategy: Optional[ast.Strategy]
-    ) -> StrategyCtx:
-        if ast_strategy is None:
-            return self._ctx.strategy
-
-        fail_fast = await ast_strategy.fail_fast.eval(self._ctx)
-        if fail_fast is None:
-            fail_fast = self._ctx.strategy.fail_fast
-        max_parallel = await ast_strategy.max_parallel.eval(self._ctx)
-        if max_parallel is None:
-            max_parallel = self._ctx.strategy.max_parallel
-
-        return StrategyCtx(fail_fast=fail_fast, max_parallel=max_parallel)
-
-    async def _setup_matrix(
-        self, ast_matrix: Optional[ast.Matrix]
-    ) -> Sequence[MatrixCtx]:
-        if ast_matrix is not None:
-            matrix = await self._matrix_init(ast_matrix)
-            matrix = await self._matrix_include(ast_matrix, matrix)
-            matrix = await self._matrix_exclude(ast_matrix, matrix)
-        else:
-            matrix = [{}]  # dummy
-        return matrix
-
-    async def _matrix_init(self, ast_matrix: ast.Matrix) -> Sequence[MatrixCtx]:
-        products = []
-        for k, lst in ast_matrix.products.items():
-            lst2 = [{k: await i.eval(self._ctx)} for i in lst]
-            products.append(lst2)
-        ret = []
-        for row in itertools.product(*products):
-            dct: Dict[str, LiteralT] = {}
-            for elem in row:
-                dct.update(elem)
-            ret.append(dct)
-        return ret
-
-    async def _matrix_exclude(
-        self, ast_matrix: ast.Matrix, matrix: Sequence[MatrixCtx]
-    ) -> Sequence[MatrixCtx]:
-        exclude = []
-        for dct in ast_matrix.exclude:
-            exclude.append({k: await v.eval(self._ctx) for k, v in dct.items()})
-        ret = []
-        for row in matrix:
-            include = True
-            for exc in exclude:
-                match = True
-                for k, v in exc.items():
-                    if row[k] != v:
-                        match = False
-                        break
-                if match:
-                    include = False
-                    break
-            if include:
-                ret.append(row)
-        return ret
-
-    async def _matrix_include(
-        self, ast_matrix: ast.Matrix, matrix: Sequence[MatrixCtx]
-    ) -> Sequence[MatrixCtx]:
-        ret = list(matrix)
-        for dct in ast_matrix.include:
-            ret.append({k: await v.eval(self._ctx) for k, v in dct.items()})
-        return ret
-
     async def _setup_ids(
-        self, ctx: BaseMatrixContext, num: int, ast_task: ast.TaskBase
+        self, ctx: MatrixOnlyContext, num: int, ast_task: ast.TaskBase
     ) -> Tuple[Optional[str], str]:
         task_id = await ast_task.id.eval(ctx)
         if task_id is None:
@@ -1541,6 +1686,61 @@ class TaskGraphBuilder:
         return default_needs
 
 
+class TaskGraphBuilder(EarlyTaskGraphBuilder):
+    MATRIX_SIZE_LIMIT = 256
+
+    def __init__(
+        self,
+        ctx: BaseBatchContext,
+        config_loader: ConfigLoader,
+        default_cache: CacheConf,
+        ast_tasks: Sequence[Union[ast.Task, ast.TaskActionCall]],
+    ):
+        super().__init__(config_loader, ast_tasks)
+        self._ctx = ctx
+        self._default_cache = default_cache
+
+    async def _extend_base(
+        self, base: BaseEarlyTask, ast_task: Union[ast.Task, ast.TaskActionCall]
+    ) -> BasePrepTask:
+        strategy = await self._setup_strategy(ast_task.strategy)
+
+        matrix_ctx = self._ctx.to_matrix_ctx(matrix=base.matrix, strategy=strategy)
+        cache = await setup_cache(
+            matrix_ctx,
+            self._default_cache,
+            ast_task.cache,
+            ast.CacheStrategy.INHERIT,
+        )
+
+        return base.to_prep_base(strategy, cache)
+
+    async def build(self) -> Mapping[str, BasePrepTask]:
+        # Super method already returns proper type (thanks to _extend_base),
+        # but it is hard to properly annotate, so we have to do runtime check here
+        ret = {}
+        tasks = await super().build()
+        for key, value in tasks.items():
+            assert isinstance(value, BasePrepTask)
+            ret[key] = value
+        return ret
+
+    async def _setup_strategy(
+        self, ast_strategy: Optional[ast.Strategy]
+    ) -> StrategyCtx:
+        if ast_strategy is None:
+            return self._ctx.strategy
+
+        fail_fast = await ast_strategy.fail_fast.eval(self._ctx)
+        if fail_fast is None:
+            fail_fast = self._ctx.strategy.fail_fast
+        max_parallel = await ast_strategy.max_parallel.eval(self._ctx)
+        if max_parallel is None:
+            max_parallel = self._ctx.strategy.max_parallel
+
+        return StrategyCtx(fail_fast=fail_fast, max_parallel=max_parallel)
+
+
 # Utils
 
 
