@@ -590,12 +590,14 @@ JOB = {
     "browse": OptBoolExpr,
     "port_forward": SimpleSeq(PortPairExpr),
     "multi": SimpleOptBoolExpr,
+    "params": None,
     **EXEC_UNIT,
 }
 
 JOB_ACTION_CALL = {
     "action": SimpleStrExpr,
     "args": SimpleMapping(StrExpr),
+    "params": None,
 }
 
 
@@ -792,20 +794,20 @@ FLOW = {
     "images": None,
     "volumes": None,
     "defaults": None,
-    "args": None,
+    "params": None,
     "jobs": None,
     "tasks": None,
 }
 
 
-ARGS = {"default": SimpleOptStrExpr, "descr": SimpleOptStrExpr}
+PARAMS = {"default": SimpleOptStrExpr, "descr": SimpleOptStrExpr}
 
 
-def parse_arg(ctor: BaseConstructor, node: yaml.MappingNode) -> ast.Arg:
-    return parse_dict(ctor, node, ARGS, ast.Arg)
+def parse_param(ctor: BaseConstructor, node: yaml.MappingNode) -> ast.Param:
+    return parse_dict(ctor, node, PARAMS, ast.Param)
 
 
-def parse_args(ctor: BaseConstructor, node: yaml.MappingNode) -> Dict[str, ast.Arg]:
+def parse_params(ctor: BaseConstructor, node: yaml.MappingNode) -> Dict[str, ast.Param]:
     ret = {}
     for k, v in node.value:
         key = ctor.construct_id(k)
@@ -813,20 +815,24 @@ def parse_args(ctor: BaseConstructor, node: yaml.MappingNode) -> Dict[str, ast.A
             default = ctor.construct_object(v)  # type: ignore[no-untyped-call]
             start = mark2pos(v.start_mark)
             end = mark2pos(v.end_mark)
-            ret[key] = ast.Arg(
+            ret[key] = ast.Param(
                 _start=start,
                 _end=end,
                 default=SimpleOptStrExpr(start, end, default),
                 descr=SimpleOptStrExpr(start, end, None),
             )
         else:
-            arg = parse_arg(ctor, v)
+            arg = parse_param(ctor, v)
             ret[key] = arg
     return ret
 
 
-FlowLoader.add_path_resolver("flow:args", [(dict, "args")])  # type: ignore
-FlowLoader.add_constructor("flow:args", parse_args)  # type: ignore
+FlowLoader.add_path_resolver("flow:params", [(dict, "params")])  # type: ignore
+FlowLoader.add_path_resolver(  # type: ignore
+    "flow:params",
+    [(dict, "jobs"), (dict, None), (dict, "params")],
+)
+FlowLoader.add_constructor("flow:params", parse_params)  # type: ignore
 
 
 def find_flow_type(
@@ -1131,6 +1137,13 @@ def preprocess_action(
             node.start_mark,
         )
 
+    if kind == ast.ActionKind.LIVE:
+        if "job" in dct and dct["job"].params is not None:
+            raise ConnectionError(
+                f"job.params is not supported inside live action, use inputs instead.",
+                node.start_mark,
+            )
+
     outputs_tmp: Optional[BatchActionOutputs] = dct.get("outputs")
     if outputs_tmp and kind != ast.ActionKind.BATCH:
         if outputs_tmp.needs is not None:
