@@ -62,6 +62,7 @@ class Bake:
     suffix: str
     # prefix -> { id -> deps }
     graphs: Mapping[FullID, Mapping[FullID, AbstractSet[FullID]]]
+    params: Optional[Mapping[str, str]]
 
     def __str__(self) -> str:
         folder = "_".join([self.batch, _dt2str(self.when), self.suffix])
@@ -140,6 +141,7 @@ class BatchStorage(abc.ABC):
             when=datetime.datetime.now(),
             suffix="suffix",
             graphs={},
+            params={},
         )
         yield bake
 
@@ -151,6 +153,7 @@ class BatchStorage(abc.ABC):
         configs_meta: Mapping[str, Any],
         configs: Sequence[ConfigFile],
         graphs: Mapping[FullID, Mapping[FullID, AbstractSet[FullID]]],
+        params: Optional[Mapping[str, str]],
     ) -> Bake:
         pass
 
@@ -540,6 +543,7 @@ class BatchFSStorage(BatchStorage):
         configs_meta: Mapping[str, Any],
         configs: Sequence[ConfigFile],
         graphs: Mapping[FullID, Mapping[FullID, AbstractSet[FullID]]],
+        params: Optional[Mapping[str, str]],
     ) -> Bake:
         when = _now()
         bake = Bake(
@@ -548,6 +552,7 @@ class BatchFSStorage(BatchStorage):
             when=when,
             suffix=secrets.token_hex(3),
             graphs=graphs,
+            params=params,
         )
         bake_uri = _mk_bake_uri(self._fs, bake)
         await self._fs.mkdir(bake_uri, parents=True)
@@ -874,6 +879,7 @@ def _bake_to_json(bake: Bake) -> Dict[str, Any]:
         "when": bake.when.isoformat(),
         "suffix": bake.suffix,
         "graphs": graphs,
+        "params": bake.params,
     }
 
 
@@ -890,6 +896,7 @@ def _bake_from_json(data: Dict[str, Any]) -> Bake:
         when=datetime.datetime.fromisoformat(data["when"]),
         suffix=data["suffix"],
         graphs=graphs,
+        params=data["params"],
     )
 
 
